AVRE = LibStub("AceAddon-3.0"):NewAddon("AVRE", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
AVRE.MINOR_VERSION = tonumber(("$Revision: 5 $"):match("%d+"))
local L = LibStub("AceLocale-3.0"):GetLocale("AVRE")
local zones = LibStub("AceLocale-3.0"):GetLocale("AVRE Zones")
local AVRE = AVRE
----



--[[
      VARIABLES ----------------------------------------------------------------------------------------------------------------
  ]]

local POLLING_FREQUENCY = 1.0

AVRE.isEnabled = false
AVRE.ver = {}
AVRE.raid = {}
AVRE.inCombat = false

AVRE.modules = {
	options = {
		loaded = false,
	},
	encounters = {
		icecrown = {
			zone = zones["Icecrown Citadel"],
			addon = "AVRE_Icecrown",
			loaded = false,
		},
	},
}

local specialEvents = {
	"PLAYER_REGEN_DISABLED",
	"PLAYER_REGEN_ENABLED",
	"CHAT_MSG_COMBAT_XP_GAIN",
	"CHAT_MSG_LOOT",
	"CHAT_MSG_COMBAT_FACTION_CHANGE",
	"CHAT_MSG_COMBAT_HONOR_GAIN",
	"CHAT_MSG_MONEY",
	"CHAT_MSG_SKILL",
}



--[[
      UPVALUES -----------------------------------------------------------------------------------------------------------------
  ]]
  
local _G = getfenv(0)
local pairs = _G.pairs
local strlower = _G.strlower
local strupper = _G.strupper
local format = _G.format



--[[
      UTILITIES ----------------------------------------------------------------------------------------------------------------
  ]]

function AVRE:tcopy(to, from)
 for k, v in pairs(from) do
  if type(v) == "table" then
   to[k] = {}
   AVRE:tcopy(to[k], v)
  else
   to[k] = v
  end
 end
end


local function colorize(s, color, highlight)
	if color and s then
		if highlight then
			return string.format("|cff%02x%02x%02x%s|r", (color.r or 1) * 128 + 127, (color.g or 1) * 128 + 127, (color.b or 1) * 128 + 127, s)
		else
			return string.format("|cff%02x%02x%02x%s|r", (color.r or 1) * 255, (color.g or 1) * 255, (color.b or 1) * 255, s)
		end
	else
		return s
	end
end



--[[
      INITIALIZATION -----------------------------------------------------------------------------------------------------------
  ]]

function AVRE:OnInitialize()
	-- Register database
 self.db = LibStub("AceDB-3.0"):New("AVREDB", self.defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	
	-- Register for slash commands
	self:RegisterChatCommand("avre", "ChatCommand")
end


function AVRE:OnEnable()
	-- Register for events
	self:UnregisterAllEvents()
	self:RegisterEvent("RAID_ROSTER_UPDATE", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnEvent")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnEvent")

	-- Update data
	self:UpdateRaid()
	
	-- Turn us on
	isEnabled = self.db.profile.enabled

	--[===[@debug@
	self:Print(L["Loaded"])
	--@end-debug@]===]
end


function AVRE:OnDisable()
	-- Cancel all timers
	self:CancelAllTimers()

	-- Turn us off
	isEnabled = false
	
	--[===[@debug@
	self:Print(L["Disabled"])
	--@end-debug@]===]
end


function AVRE:ChatCommand(input)
	if self.modules.options.loaded then InterfaceOptionsFrame_OpenToCategory(self.optionsFrame) end
end


function AVRE:OnProfileChanged(event, database, newProfileKey)
	--[===[@debug@
		self:Print(L["Profile modified, rebooting"])
	--@end-debug@]===]
 self:Disable()
 self:Enable() 
end



--[[
      EVENTS -------------------------------------------------------------------------------------------------------------------
  ]]

function AVRE:OnEvent(event, ...)

	if event == "RAID_ROSTER_UPDATE" then
		self:UpdateRaid()
		
	elseif event == "ZONE_CHANGED_NEW_AREA" then
		for k, v in pairs(self.modules.encounters) do
			if GetRealZoneText() == v.zone and v.addon and not IsAddOnLoaded(v.addon) then
				self:ScheduleTimer(function() LoadAddOn(v.addon) end, 3)
			end
		end
						
	elseif event == "PLAYER_REGEN_DISABLED" then
		if UnitInRaid("player") then
			self.inCombat = true
			self:ScheduleRepeatingTimer("OnTimer", POLLING_FREQUENCY)
		end
			
	elseif event == "PLAYER_REGEN_ENABLED" then
		self.inCombat = false
		self:CancelAllTimers()
		self:UpdateRaid()
		
 end
end


function AVRE:OnTimer()
	self:UpdateRaid()
end


function AVRE:UpdateRaid()
	if UnitInRaid("player") then
	
		for k, v in pairs(AVRE.raid) do
			AVRE.raid[k].valid = nil
		end
	
		-- Gather data about the raid
		local numRaid = GetNumRaidMembers()
		for i = 1, numRaid do
			local playerName, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
			if playerName then
				if not AVRE.raid[playerName] then AVRE.raid[playerName] = {} end
				AVRE.raid[playerName].rank = rank
				AVRE.raid[playerName].subgroup = subgroup
				AVRE.raid[playerName].level = level
				AVRE.raid[playerName].class = class
				AVRE.raid[playerName].zone = zone
				AVRE.raid[playerName].online = online
				AVRE.raid[playerName].isDead = isDead
				AVRE.raid[playerName].role = role
				AVRE.raid[playerName].isML = isML
				AVRE.raid[playerName].valid = true
			end -- valid player at that index
		end -- player loop
		
		-- Delete players no longer in the raid
		for k, v in pairs(AVRE.raid) do
			if not v.valid then
				AVRE.raid[k] = nil
			end
		end
		
	else
		AVRE.raid = {}
	end -- we're in a raid
end
